/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.objects

import com.persagy.database.SFilter
import com.persagy.database.SQueryBuilder
import com.persagy.database.enums.SOps
import com.persagy.database.extensions.keyValue
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.graphtype.RelationTypeProject
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.datacenter.wanda.rel.REq2Sy
import com.persagy.server.services.base.Service
import com.persagy.server.services.log.DataCenterLogsService
import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService

import org.slf4j.LoggerFactory

/**
 * 设备信息服务
 *
 * @author 张维新
 */
object EquipmentService : Service<Equipment>(SMybatisDao(Equipment::class.java)) {

    private val logger = LoggerFactory.getLogger(EquipmentService::class.java)

    /** 关系总览服务 */
    val relationTypeProjectservice = SBaseService(SMybatisDao(RelationTypeProject::class.java))

    /** 设备和系统关系 */
    private val rEq2SyService = SObjectService(SMybatisDao(REq2Sy::class.java))

    /**
     * 创建前赋值操作
     *
     * @param   equip   设备对象
     */
    override fun onCreateBefore(equip: Equipment): Boolean {
        /** 查询准备类型 */
        if (!equip.id.isNullOrEmpty()) {
            val isStartsWith = equip.id.toString().startsWith("Eq")
            if (!isStartsWith) {
                /** 资产id */
                equip.id = IdUtils.equipmentIdCreate(Opt.projectId!!)
            }
        } else {
            /** 资产id */
            equip.id = IdUtils.equipmentIdCreate(Opt.projectId!!)
        }
        /** 项目id */
        equip.projectId = Opt.projectId
        if (equip.classCode.isNullOrEmpty()) {
            return false
        }
        if(equip.classCode!!.length<6){
            val entity = defClassService.select(
                    SFilter.eq("projectId", equip.projectId!!),
                    SFilter.endWith("code", equip.classCode!!)
            ).entity()
            if (entity!=null){
                equip.classCode = entity.code
            } else {
                return false
            }
        }
        /** 设备名字 */
        if (equip.name.isNullOrEmpty()) {
            val entity = defClassService.select(
                SFilter.eq("projectId", equip.projectId!!),
                SFilter.eq("code", equip.classCode!!)
            ).entity()
            if (entity!=null) {
                equip.name = ShaftNameUtil.shaftName(entity.name+"-")
            }else {
                equip.name = ShaftNameUtil.shaftName("设备-")
            }
        }
        equip.objectType = "equipment"
        equip.codeType = 1
        return true
    }

    /**
     * 修改设备信息之前的操作
     */
    override fun onUpdateBefore(entity: Equipment): Boolean {
            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"update-before",entity.objectType!!,
                    entity.id!!,entity,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        return true
    }




    /**
     * 删除之前操作
     */
    override fun onDeleteBefore(entity: Equipment): Boolean {
            try {
                val equipment = this.select(SFilter.eq("id", entity.id!!)).entity()
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"delete",entity.objectType!!,
                    entity.id!!,equipment!!,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        return super.onDeleteBefore(entity)
    }


    override fun onCreateSuccess(entityList: ArrayList<Equipment>) {
        entityList.forEach {
            try {
                /** 发送消息 */
                rabbitMqService.sendObjects(Opt.projectId!!, Opt.CREATE,it.keyValue("id").toString())
            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                val entity = EquipmentService.select(SFilter.eq("id", it.keyValue("id")!!)).entity()
                if (entity != null) {
                    try {
                        /** 日志 */
                        DataCenterLogsService.addLogs(Opt.projectId!!,"create",entity.objectType!!,
                            entity.id!!,entity,Opt.account,Opt.coming)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    } //

    /**
     * 更新成功能回调函数
     *
     * @param   entityList      更新成功对象列表
     */
    override fun onUpdateSuccess(entityList: ArrayList<Equipment>) {
        try {
            for (entity in entityList) {
                try {
                    val equip = EquipmentService.select(SFilter.eq("id", entity.keyValue("id")!!)).entity()
                    if (equip != null) {
                        /** 日志 */
                        DataCenterLogsService.addLogs(Opt.projectId!!,"update-after",entity.objectType!!,
                            entity.id!!,entity,Opt.account,Opt.coming)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 发送消息 */
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.UPDATE,entity.id!!)
                } catch (e: Exception) {
                    e.toString()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * @Description:删除成功之后
     */
    override fun onDeleteSuccess(entityList: ArrayList<Equipment>) {
        super.onDeleteSuccess(entityList)
        try {
            for (entity in entityList) {
                try {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.DELETE,entity.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
}